class Constants:
    SYS_NUM_CARDINAL: str = 'builtin.num.cardinal'
    SYS_NUM_DOUBLE: str = 'builtin.num.double'
    SYS_NUM_FRACTION: str = 'builtin.num.fraction'
    SYS_NUM_INTEGER: str = 'builtin.num.integer'
    SYS_NUM: str = 'builtin.num'
    SYS_NUM_ORDINAL: str = 'builtin.num.ordinal'
    SYS_NUM_PERCENTAGE: str = 'builtin.num.percentage'

    # NARROW NO-BREAK SPACE
    NO_BREAK_SPACE: chr = '\u202f'
